#!/usr/bin/php
<?php
// A Unitrends script to use via ipmiutil getevt -s -r /path/evt-rr.php
// This parses the IPMI event severity and calls bp_send_notification().

$log= "/var/log/event-rr.log";
$prev= "/var/lib/ipmiutil/prev-rr.evt";
$cmd = basename($argv[0]);

// split_event function splits the event description into array fields
// 0=id, 1=date, 2=sev, 3=gen, 4=styp, 5=stag, 6=desc
function split_event( $str ) {
   $retarr = array();
   // parse canonical/delimited event string, if delimiter is there
   $edata = str_getcsv($str,'|');
   $i = 0;
   foreach ($edata as $efield) {
      $retarr[$i] = $efield;
      ++$i;
   }
   if ( $i < 4) {  // if no delimiter, use columns
      $retarr[0] = substr( $str, 0, 4);
      $retarr[1] = substr( $str, 5, 17);
      $retarr[2] = substr( $str, 23, 3);
      $retarr[3] = substr( $str, 27, 4);
      $tok = strtok( $str, "#");
      $retarr[4] = substr( $tok, 32, 20);
      $tok = strtok( "\n");
      $retarr[5] = "#".substr( $tok, 0, 18);
      $retarr[6] = substr( $tok, 18, 60);
   } 
   // for ($i = 0; $i <= 6; $i++) {  //++++
   //    echo "[".$i."] str(".$retarr[$i].")\n";
   // }
   return $retarr;
}

if($argc < 2) {
	echo "Usage: ".$cmd." <event string>\n";
	die;
} else {
	$evtdesc = $argv[1];
}

$when = date('Y/m/d H:i:s');
file_put_contents($log, "$when $cmd got IPMI event\n", FILE_APPEND); 
file_put_contents($log, "$evtdesc\n", FILE_APPEND); 
$evtarr = split_event($evtdesc);
$sev = trim($evtarr[2]);
// Build a unique target string with: Hostname_SensorType_SensorTag
$targ1 = trim(`uname -n`)."_".trim($evtarr[4])."_".trim($evtarr[5]);
$extra = array(" ", "#");
$target = str_replace($extra,"",$targ1);
// print "evtdesc = " .$evtdesc. "\n";
// print "sev=".$sev."\n";
// print "target=".$target."\n";

// ignore Chassis Intrusion events 
if (strstr( $evtdesc, "Chassis Intru" )) {
   file_put_contents($log, "Skipping Chasssis Intrusion\n", FILE_APPEND);
   die;
} 

// check if repeated event from $prev or not
if ( file_exists($prev) ) {
   $prevstr = file_get_contents($prev);
   $prevarr = split_event($prevstr);
   $prevpattn = $prevarr[2];
   $prevpattn .= $prevarr[3];
   $prevpattn .= $prevarr[4];
   $prevpattn .= $prevarr[5];
   $newpattn = $evtarr[2];
   $newpattn .= $evtarr[3];
   $newpattn .= $evtarr[4];
   $newpattn .= $evtarr[5];
   $ret = strcmp($prevpattn,$newpattn);
   if ( $ret == 0 ) {
       file_put_contents($log, "Skipping repeated event\n", FILE_APPEND);
       die;
   }
}

// # Map the IPMI severity to a notification_id and trapSeverity 
// # trapSeverity:  1=fatal, 2=warning, 3=notice, 0=clearing
{
     switch($sev)
     {
        case "MIN":
	    $tsev=3;
            $notif_id=155;
            break;
        case "MAJ":
	    $tsev=2;
            $notif_id=156;
            break;
        case "CRT":
	    $tsev=1;
            $notif_id=157;
            break;
	case "INF":
	    // May eventually send clearing/OK events here: $notif_id=158,sev=0
	    file_put_contents($log,"Skipping INF event\n", FILE_APPEND); 
	    die;
            break;
        default:
	    $tsev=3;
	    $notif_id=155;
            break;
     }
} 

// Send a unitrends-rr SNMP trap via bp_send_notification (needs bpdb access)
// print "Notification ID is " .$notif_id. "\n";
bp_bypass_cookie(3);
file_put_contents($log, "Calling bp_send_notification($notif_id,$target)\n", FILE_APPEND);

$ret = bp_send_notification($notif_id, $target, $evtdesc); 
if ($ret === false || $ret === NULL) {
        file_put_contents($log, bp_error(), FILE_APPEND); 
        file_put_contents($log, "\n$cmd bp_send_notification error\n", FILE_APPEND); 
} else {
        file_put_contents($log, "$cmd Success\n", FILE_APPEND); 
        file_put_contents($prev,"$evtdesc");
}

bp_destroy_cookie();
// print "Done with ".$cmd."\n";

?>
